# -*- coding: utf-8 -*-

#vérification de la version de scikit-learn
import sklearn
print(sklearn.__version__)

#modifier le working directory
import os
os.chdir("D:/DataMining/Databases_for_mining/dataset_for_soft_dev_and_comparison/arbre_scikit_learn")

#importer les données
import pandas
df = pandas.read_excel("breast.xlsx",sheet_name = 0)

#dimension du data frame
print(df.shape)

#affichage des premières lignes
print(df.head())

#information sur les variables
print(df.info())

#vérifier la distribution absolue des classes
print(df.classe.value_counts())

#la distribution relative
print(df.classe.value_counts(normalize=True))

#subdiviser les données en échantillons d'apprentissage et de test
from sklearn.model_selection import train_test_split
dfTrain, dfTest = train_test_split(df,test_size=300,random_state=1,stratify=df.classe)

#vérification des dimensions
print(dfTrain.shape)
print(dfTest.shape)

#vérification des distributions en apprentissage
print(dfTrain.classe.value_counts(normalize=True))

#vérification des distributions en test
print(dfTest.classe.value_counts(normalize=True))

#instanciation de l'arbre
from sklearn.tree import DecisionTreeClassifier
arbreFirst = DecisionTreeClassifier(min_samples_split=30,min_samples_leaf=10)

#construction de l'arbre
arbreFirst.fit(X = dfTrain.iloc[:,:-1], y = dfTrain.classe)

#affichage graphique de l'arbre - depuis sklearn 0.21
#https://scikit-learn.org/stable/modules/generated/sklearn.tree.plot_tree.html#sklearn.tree.plot_tree
from sklearn.tree import plot_tree
plot_tree(arbreFirst,feature_names = list(df.columns[:-1]),filled=True)

#affichage plus grand pour une meilleure lisibilité
import matplotlib.pyplot as plt
plt.figure(figsize=(10,10))
plot_tree(arbreFirst,feature_names = list(df.columns[:-1]),filled=True)
plt.show()

#affichage sous forme de règles
#plus facile à appréhender quand l'arbre est très grand
from sklearn.tree import export_text
tree_rules = export_text(arbreFirst,feature_names = list(df.columns[:-1]),show_weights=True)
print(tree_rules)

#importance des variables
impVarFirst = {"Variable":df.columns[:-1],"Importance":arbreFirst.feature_importances_}
print(pandas.DataFrame(impVarFirst).sort_values(by="Importance",ascending=False))

#prédiction sur l'échantillon test
predFirst = arbreFirst.predict(X=dfTest.iloc[:,:-1])

#distribution des predictions
import numpy
print(numpy.unique(predFirst,return_counts=True))

#matrice de confusion
from sklearn import metrics
print(metrics.confusion_matrix(dfTest.classe,predFirst))

#taux de reconnaissance
print(metrics.accuracy_score(dfTest.classe,predFirst))

#taux d'erreur
print(1.0 - metrics.accuracy_score(dfTest.classe,predFirst))

#rappel - sensibilité
print(metrics.recall_score(dfTest.classe,predFirst,pos_label='malignant'))

#précision
print(metrics.precision_score(dfTest.classe,predFirst,pos_label='malignant'))

#F1-score
print(metrics.f1_score(dfTest.classe,predFirst,pos_label='malignant'))

#rapport de prédiction
print(metrics.classification_report(dfTest.classe,predFirst))

#modifier les paramètres d'apprentissage
arbreSecond = DecisionTreeClassifier(min_samples_split=30,min_samples_leaf=10,max_leaf_nodes=3)

#construction de l'arbre
arbreSecond.fit(X = dfTrain.iloc[:,:-1], y = dfTrain.classe)

#affichage graphique de l'arbre
plot_tree(arbreSecond,feature_names = list(df.columns[:-1]),filled=True)

#importance des variables
impVarSecond = {"Variable":df.columns[:-1],"Importance":arbreSecond.feature_importances_}
print(pandas.DataFrame(impVarSecond).sort_values(by="Importance",ascending=False))


#prédiction sur l'échantillon test
predSecond = arbreSecond.predict(X=dfTest.iloc[:,:-1])

#matrice de confusion
print(metrics.confusion_matrix(dfTest.classe,predSecond))

#taux de reconnaissance
print(metrics.accuracy_score(dfTest.classe,predSecond))


